import 'package:equatable/equatable.dart';

import 'user_integration.dart';

class User extends Equatable {
  final bool changePassword;
  final String? photo;
  final bool admin;
  final bool allowedToChangePassword;
  final String id;
  final String username;
  final String fullName;
  final String email;
  final String locale;
  final String tenantDomain;
  final String tenantName;
  final String tenantLocale;
  final bool blocked;
  final String? authenticationType;
  final String? description;
  final List<dynamic>? properties;
  final Integration? integration;
  final String? discriminator;

  const User({
    required this.changePassword,
    this.photo,
    required this.admin,
    required this.allowedToChangePassword,
    required this.id,
    required this.username,
    required this.fullName,
    required this.email,
    required this.locale,
    required this.tenantDomain,
    required this.tenantName,
    required this.tenantLocale,
    required this.blocked,
    this.authenticationType,
    this.description,
    this.properties,
    this.integration,
    this.discriminator,
  });

  factory User.fromJson(Map<String, dynamic> json) {
    return User(
      changePassword: json['changePassword'] as bool,
      photo: json['photo'] as String,
      admin: json['admin'] as bool,
      allowedToChangePassword: json['allowedToChangePassword'] as bool,
      id: json['id'] as String,
      username: json['username'] as String,
      fullName: json['fullName'] as String,
      email: json['email'] as String,
      locale: json['locale'] as String,
      tenantDomain: json['tenantDomain'] as String,
      tenantName: json['tenantName'] as String,
      tenantLocale: json['tenantLocale'] as String,
      blocked: json['blocked'] as bool,
      authenticationType: json['authenticationType'] as String?,
      description: json['description'] as String?,
      properties: json['properties'] as List<dynamic>?,
      integration: json['integration'] == null
          ? null
          : Integration.fromJson(json['integration'] as Map<String, dynamic>),
      discriminator: json['_discriminator'] as String?,
    );
  }

  Map<String, dynamic> toJson() {
    return {
      'changePassword': changePassword,
      'photo': photo,
      'admin': admin,
      'allowedToChangePassword': allowedToChangePassword,
      'id': id,
      'username': username,
      'fullName': fullName,
      'email': email,
      'locale': locale,
      'tenantDomain': tenantDomain,
      'tenantName': tenantName,
      'tenantLocale': tenantLocale,
      'blocked': blocked,
      'authenticationType': authenticationType,
      'description': description,
      'properties': properties,
      'integration': integration?.toJson(),
      '_discriminator': discriminator,
    };
  }

  User copyWith({
    bool? changePassword,
    String? photo,
    bool? admin,
    bool? allowedToChangePassword,
    String? id,
    String? username,
    String? fullName,
    String? email,
    String? locale,
    String? tenantDomain,
    String? tenantName,
    String? tenantLocale,
    bool? blocked,
    String? authenticationType,
    String? description,
    List<String>? properties,
    Integration? integration,
    String? discriminator,
  }) {
    return User(
      changePassword: changePassword ?? this.changePassword,
      photo: photo ?? this.photo,
      admin: admin ?? this.admin,
      allowedToChangePassword:
          allowedToChangePassword ?? this.allowedToChangePassword,
      id: id ?? this.id,
      username: username ?? this.username,
      fullName: fullName ?? this.fullName,
      email: email ?? this.email,
      locale: locale ?? this.locale,
      tenantDomain: tenantDomain ?? this.tenantDomain,
      tenantName: tenantName ?? this.tenantName,
      tenantLocale: tenantLocale ?? this.tenantLocale,
      blocked: blocked ?? this.blocked,
      authenticationType: authenticationType ?? this.authenticationType,
      description: description ?? this.description,
      properties: properties ?? this.properties,
      integration: integration ?? this.integration,
      discriminator: discriminator ?? this.discriminator,
    );
  }

  @override
  bool get stringify => true;

  @override
  List<Object?> get props {
    return [
      changePassword,
      photo,
      admin,
      allowedToChangePassword,
      id,
      username,
      fullName,
      email,
      locale,
      tenantDomain,
      tenantName,
      tenantLocale,
      blocked,
      authenticationType,
      description,
      properties,
      integration,
      discriminator,
    ];
  }
}
